import argparse
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import seaborn as sns
from scipy.stats import spearmanr
from plotting import get_regplot


def load_files(annot_file, main_file):
    
    df_ratings = pd.read_csv(annot_file, sep='\t')
    df_ratings = df_ratings[['item', 'speaker', 'rating']]
    df_ratings = df_ratings.pivot(index='speaker', columns='item')

    df_main = pd.read_csv(main_file, sep='\t', index_col=0)
    
    return df_ratings, df_main


def get_ratings_distribution(df_ratings):
    
    df_ratings_counts = pd.DataFrame(df_ratings.count(), columns=['n'])\
                          .droplevel(level=0)

    # Exclude words with fewer than half retained ratings
    excl_words = df_ratings_counts.loc[df_ratings_counts['n'] < 8]\
                                  .index.to_list()
    
    # Mean number of ratings on retained items
    ratings_dist = df_ratings_counts.loc[~df_ratings_counts\
                                    .index.isin(excl_words)]\
                                    .agg(['mean', 'count'])\
                                    .round(1).to_string()
    
    return excl_words, ratings_dist


def get_model_human_corrs(df_main):
    
    corrs = []
    target_var = 'human_avg'
    for other in ['cos_avg', 'clust_cont', 'clust_reg']:
        corr, p = spearmanr(df_main[[target_var, other]])
        corrs.append([other, corr, p])
        
    # Format for output
    corrs = pd.DataFrame(corrs, columns=['estimate', 'rho', 'p'])
    corrs = corrs.set_index('estimate').round(3).to_string() 
    
    header = f'Correlations with `{target_var}` for {df_main.shape[0]} items.\n\n'
    
    corrs = header + corrs
    return corrs


def get_emp_prop_corrs(df_main):

    corrs = f'Correlation matrix for {df_main.shape[0]} items.\n\n'
    rows = ['cos_avg', 'clust_cont', 'clust_reg', 'human_avg']
    cols = ['freq', 'polys', 'specif', 'context']
    
    out = {}
    for row in rows:
        out[row] = {}
        for col in cols:
            rho, p = spearmanr(df_main[row], df_main[col])
            out[row][(col, 'rho')] = rho
            out[row][(col, 'p')] = p

    corrs += pd.DataFrame.from_dict(out, orient='index').round(3).to_string()

    return corrs


def get_quali_examples(targets, df_main_full):
    
    measures = ['human_avg', 'human_std', 'n_local', 'n_fr', 'syn_list']
    
    quali_ex = df_main_full.loc[targets][measures]\
                           .sort_values(by='human_avg', ascending=False)\
                           .round(2)

    quali_ex['syn_list'] = quali_ex['syn_list'].apply(lambda x: x.split(';')[0])

    quali_ex = quali_ex.to_string()
    
    # Number with at least one social value
    n_social = df_main_full.loc[(df_main_full['n_local'] > 0) | 
                                (df_main_full['n_fr'] > 0)].shape[0]
    
    quali_ex += f'\n\nAt least one social value for {n_social} items.'
    
    return quali_ex


def top_three_synonyms(alts):
    
    alts = alts.split('; ')[:3]
    alts = '; '.join(alts)
    return alts


def get_synonym_table(df_main_full):
    
    out_cols = ['human_avg', 'human_std', 
                'syn_list', 'syn_types', 'syn_tokens', 'syn_ttr', 
                'n_local', 'n_fr']

    df_syn = df_main_full[out_cols].copy()
    syn_out = f'Synonyms summary for {df_syn.shape[0]} items.\n\n'

    df_syn['syn_list'] = df_syn['syn_list'].apply(top_three_synonyms)

    for col in ['human_avg', 'human_std']:
        df_syn[col] = df_syn[col].round(2)

    with pd.option_context('max_colwidth', 1000):
        syn_out += df_syn[out_cols].sort_index().to_string()
    
    return syn_out


def main():
    
    
    parser = argparse.ArgumentParser()
    parser.add_argument('--annot_file', help='file containing speaker-level '
                        'annotations, default is `../speaker_annots.tsv`',
                        default='../speaker_annots.tsv')
    parser.add_argument('--shift_file', help='file containing shift-level '
                        'information, default is `../shifts_stats.tsv`',
                        default='../shifts_stats.tsv')
    parser.add_argument('--out_analysis', help='output file to write the '
                        'analysis, default is `paper_analysis.txt`',
                        default='paper_analysis.txt')
    parser.add_argument('--out_plot', help='output file to write the plot, '
                        'default is `plot.png`', default='plot.png')
    args = parser.parse_args()
    
    annot_file = args.annot_file
    main_file = args.shift_file
    analysis_file = args.out_analysis
    plot_file = args.out_plot

    output = {}
    output['Input files'] = '\n'.join([annot_file, main_file])
    
    # Load files
    df_ratings, df_main = load_files(annot_file, main_file)
    
    # Exclude words with too few ratings and get ratings distribution
    excl_words, ratings_dist = get_ratings_distribution(df_ratings)
    output['Excluded items'] = ', '.join(excl_words)
    output['Ratings distribution'] = ratings_dist
    
    # Backup full DF + filter out excluded words
    df_main_full = df_main.copy()
    df_main = df_main.loc[~df_main.index.isin(excl_words)]
    
    # Get model-human correlations
    corrs_model_human = get_model_human_corrs(df_main)
    output['Model-human correlations'] = corrs_model_human
    
    # Get empirical property correlations
    corrs_emp_prop = get_emp_prop_corrs(df_main)
    output['Empirical property correlations'] = corrs_emp_prop
    
    # Get qualitative analysis examples
    targets = ['terrace', 'boutique', 'entourage', 'definitively']
    quali_examples = get_quali_examples(targets, df_main_full)
    output['Qualitative analysis examples'] = quali_examples
    
    # Get synonym summary table
    syn_table = get_synonym_table(df_main_full)
    output['Synonym summary table'] = syn_table

    # Save plot
    get_regplot(df_main, plot_file)
    output['Regression plot'] = f'Regression plot saved to {plot_file}.'
    
    # Print output
    with open(analysis_file, 'w') as f:
        for i, header in enumerate(output):
            header_line = f'({i+1}) {header.upper()}'
            dashes = len(header_line) * '-'
            header_line = '\n'.join([dashes, header_line, dashes])
            f.write(header_line + '\n\n')
            f.write(output[header] + '\n\n\n')
    
    
if __name__ == '__main__':
    main()
    